﻿import * as THREE from "three";
import * as VerseThree from "verse-three";
import {
gui_setup,
createCube,
createSphere,
dynamicObjectBase,
param_config
} from "./student_base"


//学生が編集する静的オブジェクト処理関数
export const processStaticObject =()=>{

  //学生が記述する範囲開始
  //ここでは簡単な例として縦長の箱を表示する
  var position = {x:5,y:2,z:1};//位置
  var rotation = {x:0,y:0,z:0};//向き
  var size = {x:1,y:4,z:1};//大きさ
  var color = 0xFF9966;//色
  const cube = createCube(position,rotation,size,color);

  //ここでは重力による楕円運動の例を記述する
  var g_mass = Math.pow(10,11);//重力源の質量
  var g_rate = 6.67428*Math.pow(10,-11) ;//万有引力定数
  var r_mass = 1;//運動体の質量
  var g_center = new THREE.Vector3();
  g_center.set(5,2,1);//重力源の位置
  var r_position = new THREE.Vector3();
  r_position.set(5+2,2,1);//運動体の初期位置
  //重力加速度は
  //重力源質量 * 万有引力定数 / (距離*距離)
  //上の条件だと = 6.67428/(2*2) = 1.66857
  //円軌道でこれに釣り合う速度は
  //sqrt(重量加速度*半径) = 1.82678
  var r_velocity =new THREE.Vector3();
  r_velocity.set(0,0,1.82678);//運動体の初期速度
  //この条件だとほぼ円軌道を描く
  //ここから値をずらすと楕円軌道になる。
  //ある程度以上に大きくすると軌道を離れていく
  //小さくすると重力源に落ちて静止するはずだが実際には遠ざかる
  //これは計算精度の関係で中心近くでは非常に大きい加速が発生して
  //脱出速度を超えてしまうためと考えられる
  
  var g_accel =new THREE.Vector3();
  //運動体の表示用球
  var color = 0xff0000;
  const planet = createSphere(0.2,r_position,color);
  var diff_pos= new THREE.Vector3();
  const clock = new THREE.Clock();
  //運動処理関数
  const tick = () =>{
    const dt = clock.getDelta();//フレーム時間
    //速度*dtで位置変化
    diff_pos.copy(r_velocity);
    diff_pos.multiplyScalar(dt);
    r_position.add(diff_pos);
    planet.position.copy(r_position);

    //距離
    const r = r_position.distanceTo(g_center);
    //console.log("dist",r);
    //円軌道の場合はここの値はほぼ一定になる。
    //楕円軌道の場合は変動する

    //引力
    const f = g_rate * g_mass * r_mass / (r*r);
    //方向
    g_accel.copy(g_center);
    g_accel.sub(r_position);
    g_accel.normalize();
    //加速度ベクトル = 引力/質量
    g_accel.multiplyScalar(f/r_mass); 
    
    //加速度*dtで速度変化
    g_accel.multiplyScalar(dt); 


    r_velocity.add(g_accel);
    requestAnimationFrame(()=>{tick();});
  }
  tick();
  //学生が記述する範囲終了


}

//学生が編集する動的オブジェクト処理クラス
//ここでは例として放物線を描いて射出される球を描画する
export class dynamicObject extends dynamicObjectBase{
  //パラメータ定義 仰角 水平方向 初速の定義 defaultは初期値  min max step はuiでの値の範囲設定
  static param_define={
    elevation_angle:{
      default:45,
      min:0,
      max:90,
      step:1
    },
    direction:{
      default:0,
      min:0,
      max:360,
      step:1
    },
    power:{
      default:10,
      min:0,
      max:50,
      step:0.1
    }
  }
  
  start(param){
    //処理開始
    
    //学生が記述する範囲開始
    //この例では放物線なので
    //初期位置・初期速度・加速度を設定してフレーム処理関数を呼び出す
    const elevation_angle = param.elevation_angle / 180 *Math.PI;//仰角
    const direction = param.direction / 180 *Math.PI;//水平方向
    const power = param.power;//初速

    //初速ベクトル
    this.vx = power * Math.cos(elevation_angle)* Math.sin(direction);
    this.vy = power * Math.sin(elevation_angle);
    this.vz = power * Math.cos(elevation_angle)* Math.cos(direction);
    this.ay = -9.8;//重力加速度

    //ボールの生成
    var color = 0xffff00;
    if(param.is_self){
      color = 0x0000ff;//自分の場合だけ色を変える
    }
    this.ball = createSphere(0.1,this.base_obj.position,color);//base_objの位置をボールの初期位置とする
    
  }
  
  tick(dt){
    //学生が記述する範囲開始
    //この例では放物線なので
    //速度*フレーム時間をボールの位置に加算
    var dx = this.vx*dt;
    var dy = this.vy*dt;
    var dz = this.vz*dt;
    this.ball.position.x += dx;
    this.ball.position.y += dy;
    this.ball.position.z += dz;
    //加速度*フレーム時間を速度に加算
    this.vy += this.ay*dt;
    //この例ではyが-5未満になったら破棄する
    if(this.ball.position.y< -5){
      return false;//破棄する場合はfalseを返す。
    }
    return true;//破棄しない場合はtrueを返す
  }
  
  destroy(){
    //破棄時に呼ぶ関数
    //学生が記述する範囲開始
    //この例の場合はボールを生成したので破棄する必要がある。
    if(this.ball){
      //ボールの破棄
      this.ball.destroy();
    }
    //学生が記述する範囲終了
  }
  
  start_by_local_user(){
    //ローカルユーザーのクリック時に呼ぶ処理
    var param = {};
    param['elevation_angle'] = param_config.elevation_angle;
    param['direction'] = param_config.direction;
    param['power'] = param_config.power;
    return param;//パラメータを返す
    
  }

}


