﻿import * as THREE from "three";
import * as VerseThree from "verse-three";
import {
gui_setup,
createCube,
createSphere,
dynamicObjectBase,
param_config
} from "./student_base"


//学生が編集する静的オブジェクト処理関数
export const processStaticObject =()=>{

  //学生が記述する範囲開始
  //ここでは簡単な例として縦長の箱を表示する
  var position = {x:5,y:2,z:1};//位置
  var rotation = {x:0,y:0,z:0};//向き
  var size = {x:1,y:4,z:1};//大きさ
  var color = 0xFF9966;//色
  const cube = createCube(position,rotation,size,color);

  //学生が記述する範囲終了


}

//学生が編集する動的オブジェクト処理クラス
//ここでは例として放物線を描いて射出される球を描画する
export class dynamicObject extends dynamicObjectBase{
  //パラメータ定義 仰角 水平方向 初速の定義 defaultは初期値  min max step はuiでの値の範囲設定
  static param_define={
    elevation_angle:{
      default:45,
      min:0,
      max:90,
      step:1
    },
    direction:{
      default:0,
      min:0,
      max:360,
      step:1
    },
    power:{
      default:10,
      min:0,
      max:50,
      step:0.1
    }
  }
  
  start(param){
    //処理開始
    
    //学生が記述する範囲開始
    //この例では放物線なので
    //初期位置・初期速度・加速度を設定してフレーム処理関数を呼び出す
    const elevation_angle = param.elevation_angle / 180 *Math.PI;//仰角
    const direction = param.direction / 180 *Math.PI;//水平方向
    const power = param.power;//初速

    //初速ベクトル
    this.vx = power * Math.cos(elevation_angle)* Math.sin(direction);
    this.vy = power * Math.sin(elevation_angle);
    this.vz = power * Math.cos(elevation_angle)* Math.cos(direction);
    this.ay = -9.8;//重力加速度

    //ボールの生成
    var color = 0xffff00;
    if(param.is_self){
      color = 0x0000ff;//自分の場合だけ色を変える
    }
    this.ball = createSphere(0.1,this.base_obj.position,color);//base_objの位置をボールの初期位置とする
    
  }
  
  tick(dt){
    //学生が記述する範囲開始
    //この例では放物線なので
    //速度*フレーム時間をボールの位置に加算
    var dx = this.vx*dt;
    var dy = this.vy*dt;
    var dz = this.vz*dt;
    this.ball.position.x += dx;
    this.ball.position.y += dy;
    this.ball.position.z += dz;
    //加速度*フレーム時間を速度に加算
    this.vy += this.ay*dt;
    //この例ではyが-5未満になったら破棄する
    if(this.ball.position.y< -5){
      return false;//破棄する場合はfalseを返す。
    }
    return true;//破棄しない場合はtrueを返す
  }
  
  destroy(){
    //破棄時に呼ぶ関数
    //学生が記述する範囲開始
    //この例の場合はボールを生成したので破棄する必要がある。
    if(this.ball){
      //ボールの破棄
      this.ball.destroy();
    }
    //学生が記述する範囲終了
  }
  
  start_by_local_user(){
    //ローカルユーザーのクリック時に呼ぶ処理
    var param = {};
    param['elevation_angle'] = param_config.elevation_angle;
    param['direction'] = param_config.direction;
    param['power'] = param_config.power;
    return param;//パラメータを返す
    
  }

}


