﻿import * as THREE from "three";
import * as VerseThree from "verse-three";
import {
gui_setup,
createCube,
createSphere,
dynamicObjectBase,
param_config
} from "./student_base"
import {
trail
} from "./trail"


//学生が編集する静的オブジェクト処理関数
export const processStaticObject =()=>{

  //学生が記述する範囲開始
  //ここでは簡単な例として縦長の箱を表示する
  var position = {x:5,y:2,z:1};//位置
  var rotation = {x:0,y:0,z:0};//向き
  var size = {x:0.2,y:4,z:0.2};//大きさ
  var color = 0xFF9966;//色
  const cube = createCube(position,rotation,size,color);

  //学生が記述する範囲終了


}

//学生が編集する動的オブジェクト処理クラス
//ここでは例として放物線を描いて射出される球を描画する
export class dynamicObject extends dynamicObjectBase{
  //パラメータ定義 仰角 水平方向 初速の定義 defaultは初期値  min max step はuiでの値の範囲設定
  static param_define={
    velocity:{
      default:1.82678,
      min:0,
      max:10,
      step:0.001
    }
  }
  
  start(param){
    //処理開始
    
    //学生が記述する範囲開始

    //ここでは重力による楕円運動の例を記述する
    this.g_mass = Math.pow(10,11);//重力源の質量 １億tが非常に小さい点に存在していることになる
    this.g_rate = 6.67428*Math.pow(10,-11) ;//万有引力定数
    this.r_mass = 1;//運動体の質量 1kg
    this.g_center = new THREE.Vector3();
    this.g_center.set(5,2,1);//重力源の位置
    this.r_position = new THREE.Vector3();
    this.r_position.set(5+2,2,1);//運動体の初期位置 重力源から2m
    //重力加速度は
    //重力源質量 * 万有引力定数 / (距離*距離)
    //上の条件だと = 6.67428/(2*2) = 1.66857
    //円軌道でこれに釣り合う速度は
    //sqrt(重量加速度*半径) = 1.82678
    //この条件だとほぼ円軌道を描く
    //ここから値をずらすと楕円軌道になる。
    //ある程度以上に大きくすると軌道を離れていく
    //小さくすると重力源に落ちて静止するはずだが実際には遠ざかる
    //これは計算精度の関係で中心近くでは非常に大きい加速が発生して
    //脱出速度を超えてしまうためと考えられる

    this.r_velocity =new THREE.Vector3();
    this.r_velocity.set(0,0,param.velocity);//運動体の初期速度
    
    this.g_accel =new THREE.Vector3();
    this.diff_pos= new THREE.Vector3();

    //運動体の表示用球
    var color = 0xffff00;
    if(param.is_self){
      color = 0xff0000;//自分の場合だけ色を変える
    }

    this.planet = createSphere(0.2,this.r_position,color);
    this.trail = new trail();
    this.trail.setup(this.planet.parent,this.planet,0.02,color,1000);//軌跡
    
    //学生が記述する範囲終了




    
  }
  
  tick(dt){
    //学生が記述する範囲開始

    //速度*dtで位置変化
    this.diff_pos.copy(this.r_velocity);
    this.diff_pos.multiplyScalar(dt);
    this.r_position.add(this.diff_pos);
    this.planet.position.copy(this.r_position);
    this.trail.tick();
    
    //距離
    const r = this.r_position.distanceTo(this.g_center);
    //console.log("dist",r);
    //円軌道の場合はここの値はほぼ一定になる。
    //楕円軌道の場合は変動する


    //この例ではrが1000以上になったら破棄する
    if(r>= 1000){
      return false;//破棄する場合はfalseを返す。
    }


    //引力
    const f = this.g_rate * this.g_mass * this.r_mass / (r*r);
    //方向
    this.g_accel.copy(this.g_center);
    this.g_accel.sub(this.r_position);
    this.g_accel.normalize();
    //加速度ベクトル = 引力/質量
    this.g_accel.multiplyScalar(f/this.r_mass); 
    
    //加速度*dtで速度変化
    this.g_accel.multiplyScalar(dt); 

    this.r_velocity.add(this.g_accel);


    return true;//破棄しない場合はtrueを返す
    //学生が記述する範囲終了



  }
  
  destroy(){
    //破棄時に呼ぶ関数
    //学生が記述する範囲開始
    //この例の場合は球を生成したので破棄する必要がある。
    if(this.planet){
      //球の破棄
      this.planet.destroy();
      this.trail.destroy();
    }
    //学生が記述する範囲終了
  }
  
  start_by_local_user(){
    //ローカルユーザーのクリック時に呼ぶ処理
    var param = {};
    param['velocity'] = param_config.velocity;
    return param;//パラメータを返す
    
  }

}


