﻿import * as THREE from "three";
import * as VerseThree from "verse-three";
import {
gui_setup,
createCube,
createSphere,
dynamicObjectBase,
param_config
} from "./student_base"
import {
trail
} from "./trail"

export const shooter_conf = {
  position:{
    x:4,
    y:2,
    z:-5
  },
  rotation:{
    x:0,
    y:0,
    z:0
  },

  
};


//電流定義
export var I1 = {
  x:3,
  z:1,
  i:100,
}
export var I2 = {
  x:5,
  z:1,
  i:-100,
}


//学生が編集する静的オブジェクト処理関数
export const processStaticObject =()=>{

  //学生が記述する範囲開始
  //２本の柱
  var position1 = {x:I1.x,y:0,z:I1.z};//位置
  var position2 = {x:I2.x,y:0,z:I2.z};//位置
  var rotation = {x:0,y:0,z:0};//向き
  var size = {x:0.1,y:100,z:0.1};//大きさ
  const pillar1 = createCube(position1,rotation,size,0xff0000);
  const pillar2 = createCube(position2,rotation,size,0x0000ff);
  

  //学生が記述する範囲終了


}

//学生が編集する動的オブジェクト処理クラス
//ここでは例としてローレンツ力による挙動を示す
export class dynamicObject extends dynamicObjectBase{
  //パラメータ定義 初速の定義 defaultは初期値  min max step はuiでの値の範囲設定
  static param_define={
    "射出速度":{
      default:1.82678,
      min:0,
      max:10,
      step:0.001
    },
    "電流1":{
      default:100,
      min:-1000,
      max:1000,
      step:0.1
    },
    "電流2":{
      default:-100,
      min:-1000,
      max:1000,
      step:0.1
    },
    
  }
  
  start(param){
    //処理開始
    
    //学生が記述する範囲開始

    //ここではローレンツ力による挙動を示す
    this.r_mass = 1;//運動体の質量 1kg
    this.r_charge = 1;//運動体の電荷 1クーロン
    

    this.r_position = new THREE.Vector3();
    this.r_position.set(param.pos_x,param.pos_y,param.pos_z);
    this.r_velocity =new THREE.Vector3();
    
    this.r_accel =new THREE.Vector3();
    this.diff_pos= new THREE.Vector3();

    this.pillar1_pos= new THREE.Vector3()//距離計算用
    this.pillar2_pos= new THREE.Vector3();//距離計算用

    this.B1 = new THREE.Vector3();
    this.B2 = new THREE.Vector3();

    this.V1 = new THREE.Vector3();
    this.V2 = new THREE.Vector3();

    
    this.force1 = new THREE.Vector3();//
    this.force2 = new THREE.Vector3();//
    this.force = new THREE.Vector3();



    //運動体の表示用球
    var color = param.color;

    this.charge = createSphere(0.2,this.r_position,color);
    this.trail = new trail();
    this.trail.setup(this.charge.parent,this.charge,0.02,color,1000);//軌跡
    
    this.charge.rotation.set(param.rot_x,param.rot_y,param.rot_z);
    this.r_velocity = this.charge.getWorldDirection(this.r_velocity);
    this.r_velocity.multiplyScalar(param.velocity);
    
    //学生が記述する範囲終了


    
  }
  
  tick(dt){
    //学生が記述する範囲開始

    //速度*dtで位置変化
    this.diff_pos.copy(this.r_velocity);
    this.diff_pos.multiplyScalar(dt);
    this.r_position.add(this.diff_pos);
    this.charge.position.copy(this.r_position);
    this.trail.tick();

    //移動体
    const q=this.r_charge;//電荷
    const m=this.r_mass;//質量

    
    //電流1による力の計算
    //y軸方向の電流なので xz平面の距離をRとして
    //磁界強度は I/(2*pi*R)
    //向きは電流の軸を中心とする半径Rの円の反時計回りの接線方向

    //距離の計算
    this.pillar1_pos.set(I1.x,this.r_position.y,I1.z);
    const r1 = this.r_position.distanceTo(this.pillar1_pos);
    //この例ではrが1000以上になったら破棄する
    if(r1>= 1000){
      return false;//破棄する場合はfalseを返す。
    }
    const dx1 = this.r_position.x - this.pillar1_pos.x;
    const dz1 = this.r_position.z - this.pillar1_pos.z;
    this.B1.set(dx1,0,dz1);//電流軸から移動体までのベクトル
    this.V1.set(0,I1.i,0);//電流のベクトル
    this.B1.cross(this.V1);//外積で接線を得る
    this.B1.multiplyScalar(1/(2*Math.PI*r1*r1));//磁界ベクトル

    this.force1.copy(this.r_velocity); //移動体速度
    this.force1.multiplyScalar(q);//速度*電荷
    this.force1.cross(this.B1);//磁界との外積でローレンツ力
    
    //電流2による力の計算
    this.pillar2_pos.set(I2.x,this.r_position.y,I2.z);
    const r2 = this.r_position.distanceTo(this.pillar2_pos);
    //この例ではrが1000以上になったら破棄する
    if(r2>= 1000){
      return false;//破棄する場合はfalseを返す。
    }
    const dx2 = this.r_position.x - this.pillar2_pos.x;
    const dz2 = this.r_position.z - this.pillar2_pos.z;
    this.B2.set(dx2,0,dz2);
    this.V2.set(0,I2.i,0);
    this.B2.cross(this.V2);
    this.B2.multiplyScalar(1/(2*Math.PI*r2*r2));
    this.force2.copy(this.r_velocity); 
    this.force2.multiplyScalar(q);
    this.force2.cross(this.B2);

    
    this.force.copy(this.force1);
    this.force.add(this.force2);//force1とforce2の合成
    
    
    this.r_accel.copy(this.force);
    this.r_accel.multiplyScalar(1/m);//質量で割って加速度を得る
    this.r_accel.multiplyScalar(dt); //速度変化を求める
    this.r_velocity.add(this.r_accel);//現在速度の加算


    return true;//破棄しない場合はtrueを返す
    //学生が記述する範囲終了



  }
  
  destroy(){
    //破棄時に呼ぶ関数
    //学生が記述する範囲開始
    //この例の場合は球を生成したので破棄する必要がある。
    if(this.charge){
      //球の破棄
      this.charge.destroy();
      this.trail.destroy();
    }
    //学生が記述する範囲終了
  }
  
  start_by_local_user(){
    //ローカルユーザーのクリック時に呼ぶ処理
    var param = {};
    param['velocity'] = param_config["射出速度"];
    param['color'] = new THREE.Color(param_config["色"]).getHex();

    param['pos_x'] = param_config.pos_x;
    param['pos_y'] = param_config.pos_y;
    param['pos_z'] = param_config.pos_z;
    param['rot_x'] = param_config.rot_x;
    param['rot_y'] = param_config.rot_y;
    param['rot_z'] = param_config.rot_z;

    param['i1'] = param_config["電流1"];
    param['i2'] = param_config["電流2"];
    I1.i = param['i1'];
    I2.i = param['i2'];



    return param;//パラメータを返す
    
  }

}


