﻿import * as THREE from "three";
import * as VerseThree from "verse-three";
import { GUI } from "three/examples/jsm/libs/lil-gui.module.min.js";

const gui = new GUI({ width: 300 });
gui.open();

var scene;
export const createCube=(pos,rot,size,color)=>{
  //箱生成関数
  var isLowSpecMode = VerseThree.isLowSpecDevice();
  const cube = new THREE.Mesh(
    new THREE.BoxGeometry( size.x, size.y, size.z ),
    new THREE.MeshStandardMaterial({color: color})  
  );
  if(!isLowSpecMode){
    cube.castShadow = true;//影落とす
  }
  cube.position.set(pos.x, pos.y, pos.z);//位置指定
  cube.rotation.set(rot.x,rot.y,rot.z);
  cube.destroy = () =>{
    cube.material.dispose();
    cube.geometry.dispose();
    the_scene.remove(cube);
  };
  the_scene.add(cube);//シーンに追加
  return cube;
}


export const createSphere=(radius,pos,color)=>{
  //球生成関数
  var isLowSpecMode = VerseThree.isLowSpecDevice();
  const geometry = new THREE.SphereGeometry( radius, 32, 16 ); //半径0.1の球の形状定義
  const material = new THREE.MeshBasicMaterial( { color: color } ); //色設定
  var sphere = new THREE.Mesh( geometry, material ); //メッシュ生成
  sphere.destroy = () =>{
    sphere.material.dispose();
    sphere.geometry.dispose();
    the_scene.remove(sphere);
  };
  the_scene.add(sphere);//シーンへの追加
  sphere.position.set(pos.x,pos.y,pos.z);

  return sphere;
}

var tasks = [];
export var param_config={};

export const gui_setup = (param_define) =>{
  //学生が編集するUI設定処理
  //初期値
  for(let pname in param_define){
    param_config[pname] = param_define[pname].default;
  }
  
  //guiへの項目割り当て
  for(let pname in param_define){
    const pd = param_define[pname];
    gui.add( param_config, pname , pd.min, pd.max, pd.step );
  }

}


export class dynamicObjectBase {
  constructor(base_obj){
    this.clock = new THREE.Clock();
    this.base_obj = base_obj;
    this.id = tasks.length;
    tasks.push[this];
  }
  
  //パラメータ定義
  static param_define={
  }
  
  start(param){
  }
  
  _start(param){
    this.start(param);
    this._tick();
  }
  
  tick(dt){
  }

  _tick(){
    //フレーム処理関数
    //処理を終了して破棄する場合はdestroyを呼ぶ
    const dt = this.clock.getDelta();//フレーム時間
    
    const ret = this.tick(dt);
    if(!ret){
      this._destroy();
      return;
    };
    //次回フレームの登録
    requestAnimationFrame(()=>{this._tick();});
  }
  
  destroy(){
  }
  
  _destroy(){
    this.destroy();
    delete tasks[this.id];//参照破棄
  }

  start_by_local_user(){
  }
  
  _start_by_local_user(){
    //ローカルユーザーのクリック時に呼ぶ処理
    var param = this.start_by_local_user();
    param['is_self'] = true;
    this._start(param);//ローカル処理呼び出し
    param.ts= Date.now();//同じ値の連続送信は無視される
    param.is_self=false;
    the_player.setTextData(JSON.stringify(param));//リモートへの処理要求
    
  }

  start_by_remote_user(param){
    //リモートユーザーからの通知時に呼ぶ処理
    this._start(param);
  }

}



